
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.008.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "Plack middleware that records your client-server interactions",
  "dist_author" => [
    "Rob Hoelz <rob\@hoelz.ro>"
  ],
  "dist_name" => "Plack-Middleware-Recorder",
  "dist_version" => "0.06",
  "license" => "perl",
  "module_name" => "Plack::Middleware::Recorder",
  "recursive_test_files" => 1,
  "requires" => {
    "HTTP::Message::PSGI" => 0,
    "HTTP::Request" => 0,
    "IO::String" => 0,
    "Plack" => 0,
    "Plack::Client" => 0,
    "Plack::Middleware::Debug" => 0,
    "Scope::Guard" => 0,
    "Sereal" => 0,
    "autodie" => 0,
    "namespace::clean" => 0,
    "perl" => "v5.8.8"
  },
  "script_files" => [
    "bin/plack-record",
    "bin/plack-replay"
  ],
  "test_requires" => {
    "File::Spec" => 0,
    "HTML::Tree" => 0,
    "HTTP::Request::Common" => 0,
    "IO::Handle" => 0,
    "IPC::Open3" => 0,
    "Test::Exception" => 0,
    "Test::More" => 0,
    "Test::TCP" => 0
  }
);


my %fallback_build_requires = (
  "File::Spec" => 0,
  "HTML::Tree" => 0,
  "HTTP::Request::Common" => 0,
  "IO::Handle" => 0,
  "IPC::Open3" => 0,
  "Module::Build" => "0.28",
  "Test::Exception" => 0,
  "Test::More" => 0,
  "Test::TCP" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
